@@ -486,13 +486,36 @@ class log_monitor_manager {
         }
         
         // ProFTPD auth.log format for failed logins
-        // Format: hostname IP_ADDRESS [username] [timestamp] "PASS (hidden)" 530 -
+        // Format can be either:
+        // 1. IP_ADDRESS username - [timestamp] "PASS (hidden)" 530 -
+        // 2. hostname username - [timestamp] "PASS (hidden)" 530 -
         // Status code 530 means authentication failed
         
-        // Match failed password attempts (status code 530)
-        if (preg_match('/^(\S+)\s+(\d+\.\d+\.\d+\.\d+)\s+(?:(\S+)\s+)?.*"PASS \(hidden\)" 530/', $line, $matches)) {
-            $ip = $matches[2];
-            $username = isset($matches[3]) && $matches[3] !== '-' && $matches[3] !== 'UNKNOWN' ? $matches[3] : 'unknown';
+        // First try to extract IP from the first field (could be IP or hostname)
+        if (preg_match('/^(\S+)\s+(\S+)\s+-\s+\[.*?\]\s+"PASS \(hidden\)" 530/', $line, $matches)) {
+            $first_field = $matches[1];
+            $username = $matches[2];
+            
+            // Check if first field is an IP address
+            if (filter_var($first_field, FILTER_VALIDATE_IP)) {
+                $ip = $first_field;
+            } else {
+                // Try to extract IP from hostname patterns
+                // Common patterns: 123.45.67.89.bc.googleusercontent.com, ec2-123-45-67-89.compute-1.amazonaws.com
+                if (preg_match('/(\d+[\.\-]\d+[\.\-]\d+[\.\-]\d+)/', $first_field, $ip_matches)) {
+                    // Replace dashes with dots for EC2-style hostnames
+                    $ip = str_replace('-', '.', $ip_matches[1]);
+                    // Validate the extracted IP
+                    if (!filter_var($ip, FILTER_VALIDATE_IP)) {
+                        return false; // Can't extract valid IP
+                    }
+                } else {
+                    // Can't extract IP from hostname, skip this entry
+                    return false;
+                }
+            }
+            
+            $username = ($username !== '-' && $username !== 'UNKNOWN') ? $username : 'unknown';
             
             // Validate extracted data
             if (filter_var($ip, FILTER_VALIDATE_IP)) {
@@ -647,6 +670,16 @@ class log_monitor_manager {
         $block_duration_key = $service . '_block_duration';
         $block_duration = (int)$this->get_config($block_duration_key, 3600);
         
+        // Also block in HAProxy for full service-wide blocking
+        try {
+            require_once('/docker/whp/web/libs/haproxy_api_manager.php');
+            $haproxy = new haproxy_api_manager();
+            $haproxy->block_ip($ip, "$service: $reason", 'WHP Log Monitor');
+        } catch (Exception $e) {
+            error_log("Failed to block IP in HAProxy from log monitor: " . $e->getMessage());
+            // Continue even if HAProxy blocking fails - service block is still active
+        }
+        
         $firewall_rule_added = 0;
         $blocking_method = $this->get_config('blocking_method', 'service');
         if ($blocking_method === 'iptables') {
