@@ -159,6 +159,7 @@ class security_manager {
             return false;
         }
         
+        // Store in local database
         $stmt = $this->db->prepare('INSERT OR REPLACE INTO blocked_ips (ip_address, block_time, unblock_time, reason, failed_attempts, manual_block) VALUES (:ip, :block_time, :unblock_time, :reason, :attempts, :manual)');
         $stmt->bindValue(':ip', $ip_address, SQLITE3_TEXT);
         $stmt->bindValue(':block_time', time(), SQLITE3_INTEGER);
@@ -168,10 +169,33 @@ class security_manager {
         $stmt->bindValue(':manual', ($reason === 'Manual block') ? 1 : 0, SQLITE3_INTEGER);
         $stmt->execute();
         
+        // Also block in HAProxy for full service-wide blocking
+        try {
+            require_once('/docker/whp/web/libs/haproxy_api_manager.php');
+            $haproxy = new haproxy_api_manager();
+            $haproxy->block_ip($ip_address, $reason, 'WHP Security Manager');
+        } catch (Exception $e) {
+            error_log("Failed to block IP in HAProxy: " . $e->getMessage());
+            // Continue even if HAProxy blocking fails - local block is still active
+        }
+        
+        // Also block in SSH/FTP services via log monitor
+        try {
+            require_once('/docker/whp/web/libs/log_monitor_manager.php');
+            $log_monitor = new log_monitor_manager();
+            // Block on both SSH and FTP services
+            $log_monitor->block_service_ip('ssh', $ip_address, $reason, $failed_attempts);
+            $log_monitor->block_service_ip('ftp', $ip_address, $reason, $failed_attempts);
+        } catch (Exception $e) {
+            error_log("Failed to block IP in SSH/FTP services: " . $e->getMessage());
+            // Continue even if service blocking fails
+        }
+        
         return true;
     }
     
     public function unblock_ip($ip_address) {
+        // Remove from local database
         $stmt = $this->db->prepare('DELETE FROM blocked_ips WHERE ip_address = :ip');
         $stmt->bindValue(':ip', $ip_address, SQLITE3_TEXT);
         $stmt->execute();
@@ -181,6 +205,28 @@ class security_manager {
         $stmt->bindValue(':ip', $ip_address, SQLITE3_TEXT);
         $stmt->execute();
         
+        // Also unblock from HAProxy
+        try {
+            require_once('/docker/whp/web/libs/haproxy_api_manager.php');
+            $haproxy = new haproxy_api_manager();
+            $haproxy->unblock_ip($ip_address);
+        } catch (Exception $e) {
+            error_log("Failed to unblock IP from HAProxy: " . $e->getMessage());
+            // Continue even if HAProxy unblocking fails
+        }
+        
+        // Also unblock from SSH/FTP services
+        try {
+            require_once('/docker/whp/web/libs/log_monitor_manager.php');
+            $log_monitor = new log_monitor_manager();
+            // Unblock from both SSH and FTP services
+            $log_monitor->unblock_service_ip('ssh', $ip_address);
+            $log_monitor->unblock_service_ip('ftp', $ip_address);
+        } catch (Exception $e) {
+            error_log("Failed to unblock IP from SSH/FTP services: " . $e->getMessage());
+            // Continue even if service unblocking fails
+        }
+        
         return true;
     }
     
