@@ -11,56 +11,72 @@ $message_type = '';
 
 // Handle form submissions
 if ($_SERVER['REQUEST_METHOD'] === 'POST') {
-    if (isset($_POST['action'])) {
+    // Validate CSRF token
+    if (!validate_csrf_token()) {
+        $message = 'Invalid or expired CSRF token. Please refresh the page and try again.';
+        $message_type = 'danger';
+    } elseif (isset($_POST['action'])) {
         switch ($_POST['action']) {
             case 'unblock_ip':
-                if (!empty($_POST['ip_address'])) {
+                if (!empty($_POST['ip_address']) && filter_var($_POST['ip_address'], FILTER_VALIDATE_IP)) {
                     $result = $SecManager->unblock_ip($_POST['ip_address']);
                     if ($result) {
-                        $message = 'IP address unblocked successfully: ' . $_POST['ip_address'];
+                        $message = 'IP address unblocked successfully: ' . htmlspecialchars($_POST['ip_address']);
                         $message_type = 'success';
                     } else {
                         $message = 'Error unblocking IP address';
                         $message_type = 'danger';
                     }
+                } else {
+                    $message = 'Invalid IP address format';
+                    $message_type = 'danger';
                 }
                 break;
             case 'block_ip':
-                if (!empty($_POST['ip_address'])) {
-                    $reason = !empty($_POST['reason']) ? $_POST['reason'] : 'Manual block';
+                if (!empty($_POST['ip_address']) && filter_var($_POST['ip_address'], FILTER_VALIDATE_IP)) {
+                    $reason = !empty($_POST['reason']) ? substr(trim($_POST['reason']), 0, 255) : 'Manual block';
                     $result = $SecManager->block_ip($_POST['ip_address'], $reason);
                     if ($result) {
-                        $message = 'IP address blocked successfully: ' . $_POST['ip_address'];
+                        $message = 'IP address blocked successfully: ' . htmlspecialchars($_POST['ip_address']);
                         $message_type = 'success';
                     } else {
                         $message = 'Error blocking IP address (may be exempted)';
                         $message_type = 'danger';
                     }
+                } else {
+                    $message = 'Invalid IP address format';
+                    $message_type = 'danger';
                 }
                 break;
             case 'add_exemption':
-                if (!empty($_POST['ip_address'])) {
-                    $description = !empty($_POST['description']) ? $_POST['description'] : '';
+                if (!empty($_POST['ip_address']) && filter_var($_POST['ip_address'], FILTER_VALIDATE_IP)) {
+                    $description = !empty($_POST['description']) ? substr(trim($_POST['description']), 0, 255) : '';
                     $result = $SecManager->add_exemption_ip($_POST['ip_address'], $description);
                     if ($result) {
-                        $message = 'IP address added to exemption list: ' . $_POST['ip_address'];
+                        $message = 'IP address added to exemption list: ' . htmlspecialchars($_POST['ip_address']);
                         $message_type = 'success';
                     } else {
                         $message = 'Error adding IP to exemption list';
                         $message_type = 'danger';
                     }
+                } else {
+                    $message = 'Invalid IP address format';
+                    $message_type = 'danger';
                 }
                 break;
             case 'remove_exemption':
-                if (!empty($_POST['ip_address'])) {
+                if (!empty($_POST['ip_address']) && filter_var($_POST['ip_address'], FILTER_VALIDATE_IP)) {
                     $result = $SecManager->remove_exemption_ip($_POST['ip_address']);
                     if ($result) {
-                        $message = 'IP address removed from exemption list: ' . $_POST['ip_address'];
+                        $message = 'IP address removed from exemption list: ' . htmlspecialchars($_POST['ip_address']);
                         $message_type = 'success';
                     } else {
                         $message = 'Error removing IP from exemption list';
                         $message_type = 'danger';
                     }
+                } else {
+                    $message = 'Invalid IP address format';
+                    $message_type = 'danger';
                 }
                 break;
             case 'update_log_monitoring':
@@ -101,15 +117,24 @@ if ($_SERVER['REQUEST_METHOD'] === 'POST') {
                 }
                 break;
             case 'unblock_service_ip':
-                if (!empty($_POST['service']) && !empty($_POST['ip_address'])) {
-                    $result = $LogMonitor->unblock_service_ip($_POST['service'], $_POST['ip_address']);
-                    if ($result) {
-                        $message = 'IP address unblocked from ' . strtoupper($_POST['service']) . ' service: ' . $_POST['ip_address'];
-                        $message_type = 'success';
+                if (!empty($_POST['service']) && !empty($_POST['ip_address']) && filter_var($_POST['ip_address'], FILTER_VALIDATE_IP)) {
+                    $allowed_services = ['ssh', 'ftp'];
+                    if (in_array($_POST['service'], $allowed_services)) {
+                        $result = $LogMonitor->unblock_service_ip($_POST['service'], $_POST['ip_address']);
+                        if ($result) {
+                            $message = 'IP address unblocked from ' . strtoupper(htmlspecialchars($_POST['service'])) . ' service: ' . htmlspecialchars($_POST['ip_address']);
+                            $message_type = 'success';
+                        } else {
+                            $message = 'Error unblocking IP address from service';
+                            $message_type = 'danger';
+                        }
                     } else {
-                        $message = 'Error unblocking IP address from service';
+                        $message = 'Invalid service specified';
                         $message_type = 'danger';
                     }
+                } else {
+                    $message = 'Invalid IP address format or missing service';
+                    $message_type = 'danger';
                 }
                 break;
         }
@@ -146,18 +171,7 @@ $access_stats = $PermManager->get_system_access_stats(30);
 <?php endif; ?>
 <!-- Security Statistics -->
 <div class="row g-4 mb-4">
-    <div class="col-md-4">
-        <div class="card h-100">
-            <div class="card-header">
-                <h5 class="card-title mb-0">Blocked IPs</h5>
-            </div>
-            <div class="card-body text-center">
-                <h2 class="text-danger"><?php echo $security_stats['blocked_ips']; ?></h2>
-                <p class="mb-0">Currently blocked IP addresses</p>
-            </div>
-        </div>
-    </div>
-    <div class="col-md-4">
+    <div class="col-md-6">
         <div class="card h-100">
             <div class="card-header">
                 <h5 class="card-title mb-0">Failed Attempts (24h)</h5>
@@ -168,10 +182,10 @@ $access_stats = $PermManager->get_system_access_stats(30);
             </div>
         </div>
     </div>
-    <div class="col-md-4">
+    <div class="col-md-6">
         <div class="card h-100">
             <div class="card-header">
-                <h5 class="card-title mb-0">Exempted IPs</h5>
+                <h5 class="card-title mb-0">Exempt IPs</h5>
             </div>
             <div class="card-body text-center">
                 <h2 class="text-success"><?php echo $security_stats['exemption_ips']; ?></h2>
@@ -181,23 +195,23 @@ $access_stats = $PermManager->get_system_access_stats(30);
     </div>
 </div>
 
-<!-- SSH/FTP Service Statistics -->
+<!-- Service Monitoring Statistics -->
 <div class="row g-4 mb-4">
-    <div class="col-md-3">
+    <div class="col-md-4">
         <div class="card h-100">
             <div class="card-header">
-                <h5 class="card-title mb-0">SSH Blocked IPs</h5>
+                <h5 class="card-title mb-0">Total Blocked IPs</h5>
             </div>
             <div class="card-body text-center">
-                <h2 class="text-danger"><?php echo $service_stats['ssh']['blocked_ips']; ?></h2>
-                <p class="mb-0">Currently blocked from SSH</p>
-                <?php if (!$service_stats['ssh']['monitoring_enabled']): ?>
-                <small class="text-muted">(Monitoring disabled)</small>
-                <?php endif; ?>
+                <?php 
+                $total_blocked = $service_stats['ssh']['blocked_ips'] + $service_stats['ftp']['blocked_ips'] + $security_stats['blocked_ips'];
+                ?>
+                <h2 class="text-danger"><?php echo $total_blocked; ?></h2>
+                <p class="mb-0">Currently blocked across all services</p>
             </div>
         </div>
     </div>
-    <div class="col-md-3">
+    <div class="col-md-4">
         <div class="card h-100">
             <div class="card-header">
                 <h5 class="card-title mb-0">SSH Failed Attempts (24h)</h5>
@@ -205,24 +219,13 @@ $access_stats = $PermManager->get_system_access_stats(30);
             <div class="card-body text-center">
                 <h2 class="text-warning"><?php echo $service_stats['ssh']['failed_attempts_24h']; ?></h2>
                 <p class="mb-0">SSH login failures</p>
-            </div>
-        </div>
-    </div>
-    <div class="col-md-3">
-        <div class="card h-100">
-            <div class="card-header">
-                <h5 class="card-title mb-0">FTP Blocked IPs</h5>
-            </div>
-            <div class="card-body text-center">
-                <h2 class="text-danger"><?php echo $service_stats['ftp']['blocked_ips']; ?></h2>
-                <p class="mb-0">Currently blocked from FTP</p>
-                <?php if (!$service_stats['ftp']['monitoring_enabled']): ?>
+                <?php if (!$service_stats['ssh']['monitoring_enabled']): ?>
                 <small class="text-muted">(Monitoring disabled)</small>
                 <?php endif; ?>
             </div>
         </div>
     </div>
-    <div class="col-md-3">
+    <div class="col-md-4">
         <div class="card h-100">
             <div class="card-header">
                 <h5 class="card-title mb-0">FTP Failed Attempts (24h)</h5>
@@ -230,6 +233,9 @@ $access_stats = $PermManager->get_system_access_stats(30);
             <div class="card-body text-center">
                 <h2 class="text-warning"><?php echo $service_stats['ftp']['failed_attempts_24h']; ?></h2>
                 <p class="mb-0">FTP login failures</p>
+                <?php if (!$service_stats['ftp']['monitoring_enabled']): ?>
+                <small class="text-muted">(Monitoring disabled)</small>
+                <?php endif; ?>
             </div>
         </div>
     </div>
@@ -451,7 +457,7 @@ $access_stats = $PermManager->get_system_access_stats(30);
 
 <div class="row g-4 mb-4">
     <!-- Block IP Panel -->
-    <div class="col-md-6">
+    <div class="col-md-4">
         <div class="card h-100">
             <div class="card-header">
                 <h5 class="card-title mb-0">Block IP Address</h5>
@@ -474,7 +480,7 @@ $access_stats = $PermManager->get_system_access_stats(30);
         </div>
     </div>
     <!-- Add to Exemption List Panel -->
-    <div class="col-md-6">
+    <div class="col-md-8">
         <div class="card h-100">
             <div class="card-header">
                 <h5 class="card-title mb-0">Add to Exemption List</h5>
@@ -483,13 +489,19 @@ $access_stats = $PermManager->get_system_access_stats(30);
                 <form method="POST" action="">
                     <input type="hidden" name="csrf_token" value="<?php echo get_csrf_token(); ?>">
                     <input type="hidden" name="action" value="add_exemption">
-                    <div class="mb-3">
-                        <label for="exemption_ip" class="form-label">IP Address</label>
-                        <input type="text" class="form-control" id="exemption_ip" name="ip_address" placeholder="192.168.1.100" required>
-                    </div>
-                    <div class="mb-3">
-                        <label for="exemption_description" class="form-label">Description (optional)</label>
-                        <input type="text" class="form-control" id="exemption_description" name="description" placeholder="Office, Home, etc.">
+                    <div class="row">
+                        <div class="col-md-6">
+                            <div class="mb-3">
+                                <label for="exemption_ip" class="form-label">IP Address</label>
+                                <input type="text" class="form-control" id="exemption_ip" name="ip_address" placeholder="192.168.1.100" required>
+                            </div>
+                        </div>
+                        <div class="col-md-6">
+                            <div class="mb-3">
+                                <label for="exemption_description" class="form-label">Description (optional)</label>
+                                <input type="text" class="form-control" id="exemption_description" name="description" placeholder="Office, Home, etc.">
+                            </div>
+                        </div>
                     </div>
                     <button type="submit" class="btn btn-success">Add to Exemption List</button>
                 </form>
@@ -534,7 +546,7 @@ $access_stats = $PermManager->get_system_access_stats(30);
     <div class="col-md-6">
         <div class="card h-100">
             <div class="card-header">
-                <h5 class="card-title mb-0">Exempted IPs</h5>
+                <h5 class="card-title mb-0">Exempt IPs</h5>
             </div>
             <div class="card-body">
                 <div class="table-responsive">
@@ -566,12 +578,42 @@ $access_stats = $PermManager->get_system_access_stats(30);
 </div>
 
 <!-- Service Blocked IPs -->
-<?php if (!empty($service_blocked_ips)): ?>
+<?php 
+// Pagination setup
+$page = isset($_GET['page']) ? max(1, intval($_GET['page'])) : 1;
+$search = isset($_GET['search']) ? trim($_GET['search']) : '';
+$per_page = 25;
+
+// Filter IPs based on search
+$filtered_ips = $service_blocked_ips;
+if (!empty($search)) {
+    $filtered_ips = array_filter($service_blocked_ips, function($ip) use ($search) {
+        return stripos($ip['ip_address'], $search) !== false;
+    });
+}
+
+$total_ips = count($filtered_ips);
+$total_pages = ceil($total_ips / $per_page);
+$page = min($page, $total_pages ?: 1);
+$offset = ($page - 1) * $per_page;
+
+// Get current page items
+$current_page_ips = array_slice($filtered_ips, $offset, $per_page);
+
+if (!empty($service_blocked_ips)): ?>
 <div class="row g-4 mb-4">
     <div class="col-md-12">
         <div class="card">
-            <div class="card-header">
-                <h5 class="card-title mb-0">Service Blocked IPs (SSH/FTP)</h5>
+            <div class="card-header d-flex justify-content-between align-items-center">
+                <h5 class="card-title mb-0">Blocked IPs</h5>
+                <form method="GET" action="" class="d-flex">
+                    <input type="text" class="form-control form-control-sm me-2" name="search" 
+                           placeholder="Search IP..." value="<?php echo htmlspecialchars($search); ?>" style="width: 200px;">
+                    <button type="submit" class="btn btn-sm btn-primary">Search</button>
+                    <?php if (!empty($search)): ?>
+                    <a href="?" class="btn btn-sm btn-secondary ms-2">Clear</a>
+                    <?php endif; ?>
+                </form>
             </div>
             <div class="card-body">
                 <div class="table-responsive">
@@ -588,7 +630,14 @@ $access_stats = $PermManager->get_system_access_stats(30);
                             </tr>
                         </thead>
                         <tbody>
-                            <?php foreach ($service_blocked_ips as $blocked): ?>
+                            <?php if (empty($current_page_ips)): ?>
+                            <tr>
+                                <td colspan="7" class="text-center text-muted">
+                                    <?php echo !empty($search) ? 'No IPs found matching your search' : 'No blocked IPs'; ?>
+                                </td>
+                            </tr>
+                            <?php else: ?>
+                            <?php foreach ($current_page_ips as $blocked): ?>
                             <tr>
                                 <td><span class="badge bg-<?php echo $blocked['service_type'] == 'ssh' ? 'primary' : 'info'; ?>"><?php echo strtoupper($blocked['service_type']); ?></span></td>
                                 <td><code><?php echo htmlspecialchars($blocked['ip_address']); ?></code></td>
@@ -639,9 +688,63 @@ $access_stats = $PermManager->get_system_access_stats(30);
                                 </td>
                             </tr>
                             <?php endforeach; ?>
+                            <?php endif; ?>
                         </tbody>
                     </table>
                 </div>
+                
+                <?php if ($total_pages > 1): ?>
+                <!-- Pagination -->
+                <nav aria-label="Blocked IPs pagination">
+                    <ul class="pagination justify-content-center mb-0">
+                        <?php if ($page > 1): ?>
+                        <li class="page-item">
+                            <a class="page-link" href="?page=<?php echo $page - 1; ?>&search=<?php echo urlencode($search); ?>">Previous</a>
+                        </li>
+                        <?php endif; ?>
+                        
+                        <?php
+                        $start_page = max(1, $page - 2);
+                        $end_page = min($total_pages, $page + 2);
+                        
+                        if ($start_page > 1): ?>
+                        <li class="page-item">
+                            <a class="page-link" href="?page=1&search=<?php echo urlencode($search); ?>">1</a>
+                        </li>
+                        <?php if ($start_page > 2): ?>
+                        <li class="page-item disabled"><span class="page-link">...</span></li>
+                        <?php endif; ?>
+                        <?php endif; ?>
+                        
+                        <?php for ($i = $start_page; $i <= $end_page; $i++): ?>
+                        <li class="page-item <?php echo $i == $page ? 'active' : ''; ?>">
+                            <a class="page-link" href="?page=<?php echo $i; ?>&search=<?php echo urlencode($search); ?>"><?php echo $i; ?></a>
+                        </li>
+                        <?php endfor; ?>
+                        
+                        <?php if ($end_page < $total_pages): ?>
+                        <?php if ($end_page < $total_pages - 1): ?>
+                        <li class="page-item disabled"><span class="page-link">...</span></li>
+                        <?php endif; ?>
+                        <li class="page-item">
+                            <a class="page-link" href="?page=<?php echo $total_pages; ?>&search=<?php echo urlencode($search); ?>"><?php echo $total_pages; ?></a>
+                        </li>
+                        <?php endif; ?>
+                        
+                        <?php if ($page < $total_pages): ?>
+                        <li class="page-item">
+                            <a class="page-link" href="?page=<?php echo $page + 1; ?>&search=<?php echo urlencode($search); ?>">Next</a>
+                        </li>
+                        <?php endif; ?>
+                    </ul>
+                </nav>
+                <?php endif; ?>
+                
+                <?php if (!empty($search)): ?>
+                <div class="text-center mt-2">
+                    <small class="text-muted">Showing <?php echo $total_ips; ?> result(s) for "<?php echo htmlspecialchars($search); ?>"</small>
+                </div>
+                <?php endif; ?>
             </div>
         </div>
     </div>
